from PySide.QtCore import *
from PySide.QtGui  import *

from .notmuch_cli import *

class Message(QObject):
    def __init__(self, level, data):
        self.level   = level
        self.headers = data["headers"]
        
        self.fro = data["headers"]["From"]
        # is it really correct ?
        self.fro = self.fro.replace("<", "&lt;")
        self.fro = self.fro.replace(">", "&gt;")
        
        self.payload = data["body"]
        self.body    = data["body"][0]["content"]
        
        self.date = datetime.fromtimestamp(data["timestamp"])
        self.tags = " ".join(data["tags"])


class ThreadModel(QAbstractListModel):
    SUBJECT_ROLE = Qt.UserRole + 1
    DATE_ROLE    = Qt.UserRole + 2
    TO_ROLE      = Qt.UserRole + 3
    FROM_ROLE    = Qt.UserRole + 4
    TAGS_ROLE    = Qt.UserRole + 5
    COLOR_ROLE   = Qt.UserRole + 6
    BODY_ROLE    = Qt.UserRole + 7
    LEVEL_ROLE    = Qt.UserRole + 8
    
    def __init__(self, parent=None):
        super(ThreadModel, self).__init__(parent)
        self._data = []
        
        keys = {}
        keys[self.SUBJECT_ROLE] = 'subject'
        keys[self.DATE_ROLE   ] = 'date'
        keys[self.TO_ROLE     ] = 'to'
        keys[self.FROM_ROLE   ] = 'from'
        keys[self.TAGS_ROLE   ] = 'tags'
        keys[self.COLOR_ROLE  ] = 'thread_color'
        keys[self.BODY_ROLE   ] = 'body'
        keys[self.LEVEL_ROLE  ] = 'level'
        self.setRoleNames(keys)
    
    @Slot(str)
    def show(self, thread_id):
        show = Show(thread_id)
        
        self.beginResetModel()
        self._data = []
        for (level, msg) in show.walk():
            self._data.append(Message(level, msg))
        self.endResetModel()
    
    def rowCount(self, index):
        return len(self._data)
    
    def data(self, index, role):
        if not index.isValid():
            return None
        
        if index.row() > len(self._data):
            return None
        
        line = self._data[index.row()]

        if role == self.SUBJECT_ROLE:
            return line.headers["Subject"]
        elif role == self.DATE_ROLE:
            return str(line.date)
        elif role == self.FROM_ROLE:
            return line.fro
        elif role == self.TO_ROLE:
            return line.headers["To"]
        elif role == self.TAGS_ROLE:
            return line.tags
        elif role == self.COLOR_ROLE:
            return line.color
        elif role == self.BODY_ROLE:
            return line.body
        elif role == self.LEVEL_ROLE:
            return line.level
        else:
            return None

